@@ -3,7 +3,7 @@
 #include <QMenu>
 #include <QObject>
 
-#include "preferences/usersettings.h"
+class ConfigKey;
 
 class ControlPickerMenu : public QMenu {
     Q_OBJECT
@@ -36,7 +36,7 @@ class ControlPickerMenu : public QMenu {
     void controlChosen(int controlIndex);
 
   private:
-    QMenu* addSubmenu(QString title, QMenu* pParent=NULL);
+    QMenu* addSubmenu(QString title, QMenu* pParent = NULL);
     void addSingleControl(const QString& group,
             const QString& control,
             const QString& title,
@@ -103,7 +103,7 @@ class ControlPickerMenu : public QMenu {
 
     int addAvailableControl(const ConfigKey& key, const QString& title, const QString& description);
 
-    QString m_effectMasterOutputStr;
+    QString m_effectMainOutputStr;
     QString m_effectHeadphoneOutputStr;
     QString m_deckStr;
     QString m_previewdeckStr;
@@ -115,6 +115,7 @@ class ControlPickerMenu : public QMenu {
     QString m_effectUnitStr;
     QString m_effectStr;
     QString m_parameterStr;
+    QString m_buttonParameterStr;
     QString m_libraryStr;
 
     QList<ConfigKey> m_controlsAvailable;
