@@ -27,8 +27,12 @@
 #include "waveform/widgets/allshader/filteredwaveformwidget.h"
 #include "waveform/widgets/allshader/hsvwaveformwidget.h"
 #include "waveform/widgets/allshader/lrrgbwaveformwidget.h"
+#include "waveform/widgets/allshader/rgbstackedwaveformwidget.h"
 #include "waveform/widgets/allshader/rgbwaveformwidget.h"
 #include "waveform/widgets/allshader/simplewaveformwidget.h"
+#include "waveform/widgets/allshader/waveformwidgettexturedfiltered.h"
+#include "waveform/widgets/allshader/waveformwidgettexturedrgb.h"
+#include "waveform/widgets/allshader/waveformwidgettexturedstacked.h"
 #else
 #include "waveform/widgets/qthsvwaveformwidget.h"
 #include "waveform/widgets/qtrgbwaveformwidget.h"
@@ -36,12 +40,12 @@
 #include "waveform/widgets/qtvsynctestwidget.h"
 #include "waveform/widgets/qtwaveformwidget.h"
 #endif
+#include "waveform/widgets/deprecated/glrgbwaveformwidget.h"
+#include "waveform/widgets/deprecated/glsimplewaveformwidget.h"
+#include "waveform/widgets/deprecated/glslwaveformwidget.h"
+#include "waveform/widgets/deprecated/glvsynctestwidget.h"
+#include "waveform/widgets/deprecated/glwaveformwidget.h"
 #include "waveform/widgets/emptywaveformwidget.h"
-#include "waveform/widgets/glrgbwaveformwidget.h"
-#include "waveform/widgets/glsimplewaveformwidget.h"
-#include "waveform/widgets/glslwaveformwidget.h"
-#include "waveform/widgets/glvsynctestwidget.h"
-#include "waveform/widgets/glwaveformwidget.h"
 #include "waveform/widgets/hsvwaveformwidget.h"
 #include "waveform/widgets/rgbwaveformwidget.h"
 #include "waveform/widgets/softwarewaveformwidget.h"
@@ -51,6 +55,26 @@
 #include "widget/wwaveformviewer.h"
 
 namespace {
+bool isDeprecated(WaveformWidgetType::Type t) {
+    switch (t) {
+    case WaveformWidgetType::GLRGBWaveform:
+        return true;
+    case WaveformWidgetType::GLSimpleWaveform:
+        return true;
+    case WaveformWidgetType::GLSLFilteredWaveform:
+        return true;
+    case WaveformWidgetType::GLSLRGBWaveform:
+        return true;
+    case WaveformWidgetType::GLSLRGBStackedWaveform:
+        return true;
+    case WaveformWidgetType::GLVSyncTest:
+        return true;
+    case WaveformWidgetType::GLFilteredWaveform:
+        return true;
+    default:
+        return false;
+    }
+}
 // Returns true if the given waveform should be rendered.
 bool shouldRenderWaveform(WaveformWidgetAbstract* pWaveformWidget) {
     if (pWaveformWidget == nullptr ||
@@ -111,6 +135,11 @@ WaveformWidgetFactory::WaveformWidgetFactory()
           m_defaultZoom(WaveformWidgetRenderer::s_waveformDefaultZoom),
           m_zoomSync(true),
           m_overviewNormalized(false),
+          m_untilMarkShowBeats(false),
+          m_untilMarkShowTime(false),
+          m_untilMarkAlign(Qt::AlignVCenter),
+          m_untilMarkTextPointSize(24),
+          m_untilMarkTextHeightLimit(toUntilMarkTextHeightLimit(0)),
           m_openGlAvailable(false),
           m_openGlesAvailable(false),
           m_openGLShaderAvailable(false),
@@ -403,6 +432,37 @@ bool WaveformWidgetFactory::setConfig(UserSettingsPointer config) {
             WaveformWidgetRenderer::s_defaultPlayMarkerPosition);
     setPlayMarkerPosition(m_playMarkerPosition);
 
+    int untilMarkShowBeats =
+            m_config->getValueString(
+                            ConfigKey("[Waveform]", "UntilMarkShowBeats"))
+                    .toInt(&ok);
+    if (ok) {
+        setUntilMarkShowBeats(static_cast<bool>(untilMarkShowBeats));
+    } else {
+        m_config->set(ConfigKey("[Waveform]", "UntilMarkShowBeats"),
+                ConfigValue(m_untilMarkShowBeats));
+    }
+    int untilMarkShowTime =
+            m_config->getValueString(
+                            ConfigKey("[Waveform]", "UntilMarkShowTime"))
+                    .toInt(&ok);
+    if (ok) {
+        setUntilMarkShowTime(static_cast<bool>(untilMarkShowTime));
+    } else {
+        m_config->set(ConfigKey("[Waveform]", "UntilMarkShowTime"),
+                ConfigValue(m_untilMarkShowTime));
+    }
+
+    setUntilMarkAlign(toUntilMarkAlign(
+            m_config->getValue(ConfigKey("[Waveform]", "UntilMarkAlign"),
+                    toUntilMarkAlignIndex(m_untilMarkAlign))));
+    setUntilMarkTextPointSize(
+            m_config->getValue(ConfigKey("[Waveform]", "UntilMarkTextPointSize"),
+                    m_untilMarkTextPointSize));
+    setUntilMarkTextHeightLimit(toUntilMarkTextHeightLimit(
+            m_config->getValue(ConfigKey("[Waveform]", "UntilMarkTextHeightLimit"),
+                    toUntilMarkTextHeightLimitIndex(m_untilMarkTextHeightLimit))));
+
     return true;
 }
 
@@ -519,25 +579,38 @@ bool WaveformWidgetFactory::setWidgetType(
 
     // check if type is acceptable
     int index = findHandleIndexFromType(type);
-    if (index > -1) {
-        // type is acceptable
-        *pCurrentType = type;
-        if (m_config) {
-            m_configType = type;
-            m_config->setValue(
-                    ConfigKey("[Waveform]", "WaveformType"),
-                    static_cast<int>(*pCurrentType));
-        }
-        return true;
-    }
-
-    // fallback
-    *pCurrentType = WaveformWidgetType::EmptyWaveform;
+    bool isAcceptable = index > -1;
+    *pCurrentType = isAcceptable ? type : WaveformWidgetType::EmptyWaveform;
     if (m_config) {
         m_configType = *pCurrentType;
         m_config->setValue(
-                ConfigKey("[Waveform]", "WaveformType"),
-                static_cast<int>(*pCurrentType));
+                ConfigKey("[Waveform]", "WaveformType"), *pCurrentType);
+    }
+    return isAcceptable;
+}
+
+bool WaveformWidgetFactory::widgetTypeSupportsUntilMark() const {
+    switch (m_configType) {
+    case WaveformWidgetType::AllShaderRGBWaveform:
+        return true;
+    case WaveformWidgetType::AllShaderLRRGBWaveform:
+        return true;
+    case WaveformWidgetType::AllShaderFilteredWaveform:
+        return true;
+    case WaveformWidgetType::AllShaderSimpleWaveform:
+        return true;
+    case WaveformWidgetType::AllShaderHSVWaveform:
+        return true;
+    case WaveformWidgetType::AllShaderRGBStackedWaveform:
+        return true;
+    case WaveformWidgetType::AllShaderTexturedFiltered:
+        return true;
+    case WaveformWidgetType::AllShaderTexturedRGB:
+        return true;
+    case WaveformWidgetType::AllShaderTexturedStacked:
+        return true;
+    default:
+        break;
     }
     return false;
 }
@@ -649,6 +722,9 @@ void WaveformWidgetFactory::setVisualGain(FilterIndex index, double gain) {
     if (m_config) {
         m_config->set(ConfigKey("[Waveform]","VisualGain_" + QString::number(index)), QString::number(m_visualGain[index]));
     }
+    if (!m_overviewNormalized && index == FilterIndex::All) {
+        emit overallVisualGainChanged();
+    }
 }
 
 double WaveformWidgetFactory::getVisualGain(FilterIndex index) const {
@@ -660,6 +736,7 @@ void WaveformWidgetFactory::setOverviewNormalized(bool normalize) {
     if (m_config) {
         m_config->set(ConfigKey("[Waveform]","OverviewNormalized"), ConfigValue(m_overviewNormalized));
     }
+    emit overviewNormalizeChanged();
 }
 
 void WaveformWidgetFactory::setPlayMarkerPosition(double position) {
@@ -688,7 +765,7 @@ void WaveformWidgetFactory::notifyZoomChange(WWaveformViewer* viewer) {
 }
 
 void WaveformWidgetFactory::renderSelf() {
-    ScopedTimer t("WaveformWidgetFactory::render() %1waveforms",
+    ScopedTimer t(u"WaveformWidgetFactory::render() %1waveforms",
             static_cast<int>(m_waveformWidgetHolders.size()));
 
     if (!m_skipRender) {
@@ -762,7 +839,7 @@ void WaveformWidgetFactory::render() {
 }
 
 void WaveformWidgetFactory::swapSelf() {
-    ScopedTimer t("WaveformWidgetFactory::swap() %1waveforms",
+    ScopedTimer t(u"WaveformWidgetFactory::swap() %1waveforms",
             static_cast<int>(m_waveformWidgetHolders.size()));
 
     // Do this in an extra slot to be sure to hit the desired interval
@@ -966,6 +1043,13 @@ void WaveformWidgetFactory::evaluateWidgets() {
 #else
             setWaveformVarsByType.operator()<allshader::FilteredWaveformWidget>();
             break;
+#endif
+        case WaveformWidgetType::AllShaderRGBStackedWaveform:
+#ifndef MIXXX_USE_QOPENGL
+            continue;
+#else
+            setWaveformVarsByType.operator()<allshader::RGBStackedWaveformWidget>();
+            break;
 #endif
         case WaveformWidgetType::AllShaderSimpleWaveform:
 #ifndef MIXXX_USE_QOPENGL
@@ -980,13 +1064,34 @@ void WaveformWidgetFactory::evaluateWidgets() {
 #else
             setWaveformVarsByType.operator()<allshader::HSVWaveformWidget>();
             break;
+#endif
+        case WaveformWidgetType::AllShaderTexturedFiltered:
+#ifndef MIXXX_USE_QOPENGL
+            continue;
+#else
+            setWaveformVarsByType.operator()<allshader::WaveformWidgetTexturedFiltered>();
+            break;
+#endif
+        case WaveformWidgetType::AllShaderTexturedRGB:
+#ifndef MIXXX_USE_QOPENGL
+            continue;
+#else
+            setWaveformVarsByType.operator()<allshader::WaveformWidgetTexturedRGB>();
+            break;
+#endif
+        case WaveformWidgetType::AllShaderTexturedStacked:
+#ifndef MIXXX_USE_QOPENGL
+            continue;
+#else
+            setWaveformVarsByType.operator()<allshader::WaveformWidgetTexturedStacked>();
+            break;
 #endif
         default:
             DEBUG_ASSERT(!"Unexpected WaveformWidgetType");
             continue;
         }
 
-        bool active = true;
+        bool active = !isDeprecated(static_cast<WaveformWidgetType::Type>(type));
         if (isOpenGlAvailable()) {
             if (useOpenGles && !useOpenGl) {
                 active = false;
@@ -1071,12 +1176,24 @@ WaveformWidgetAbstract* WaveformWidgetFactory::createWaveformWidget(
         case WaveformWidgetType::AllShaderFilteredWaveform:
             widget = new allshader::FilteredWaveformWidget(viewer->getGroup(), viewer);
             break;
+        case WaveformWidgetType::AllShaderRGBStackedWaveform:
+            widget = new allshader::RGBStackedWaveformWidget(viewer->getGroup(), viewer);
+            break;
         case WaveformWidgetType::AllShaderSimpleWaveform:
             widget = new allshader::SimpleWaveformWidget(viewer->getGroup(), viewer);
             break;
         case WaveformWidgetType::AllShaderHSVWaveform:
             widget = new allshader::HSVWaveformWidget(viewer->getGroup(), viewer);
             break;
+        case WaveformWidgetType::AllShaderTexturedFiltered:
+            widget = new allshader::WaveformWidgetTexturedFiltered(viewer->getGroup(), viewer);
+            break;
+        case WaveformWidgetType::AllShaderTexturedRGB:
+            widget = new allshader::WaveformWidgetTexturedRGB(viewer->getGroup(), viewer);
+            break;
+        case WaveformWidgetType::AllShaderTexturedStacked:
+            widget = new allshader::WaveformWidgetTexturedStacked(viewer->getGroup(), viewer);
+            break;
 #else
         case WaveformWidgetType::QtSimpleWaveform:
             widget = new QtSimpleWaveformWidget(viewer->getGroup(), viewer);
@@ -1215,6 +1332,9 @@ QString WaveformWidgetFactory::buildWidgetDisplayName() const {
             extras.push_back(QStringLiteral("GL"));
         }
     }
+    if (WaveformT::useTextureForWaveform()) {
+        extras.push_back(QStringLiteral("high detail"));
+    }
     QString name = WaveformT::getWaveformWidgetName();
     if (extras.isEmpty()) {
         return name;
@@ -1261,3 +1381,94 @@ QSurfaceFormat WaveformWidgetFactory::getSurfaceFormat(UserSettingsPointer confi
 #endif
     return format;
 }
+
+void WaveformWidgetFactory::setUntilMarkShowBeats(bool value) {
+    m_untilMarkShowBeats = value;
+    if (m_config) {
+        m_config->set(ConfigKey("[Waveform]", "UntilMarkShowBeats"),
+                ConfigValue(m_untilMarkShowBeats));
+    }
+}
+
+void WaveformWidgetFactory::setUntilMarkShowTime(bool value) {
+    m_untilMarkShowTime = value;
+    if (m_config) {
+        m_config->set(ConfigKey("[Waveform]", "UntilMarkShowTime"),
+                ConfigValue(m_untilMarkShowTime));
+    }
+}
+
+void WaveformWidgetFactory::setUntilMarkAlign(Qt::Alignment align) {
+    m_untilMarkAlign = align;
+    if (m_config) {
+        m_config->setValue(ConfigKey("[Waveform]", "UntilMarkAlign"),
+                toUntilMarkAlignIndex(m_untilMarkAlign));
+    }
+}
+
+void WaveformWidgetFactory::setUntilMarkTextPointSize(int value) {
+    m_untilMarkTextPointSize = value;
+    if (m_config) {
+        m_config->setValue(ConfigKey("[Waveform]", "UntilMarkTextPointSize"),
+                m_untilMarkTextPointSize);
+    }
+}
+
+void WaveformWidgetFactory::setUntilMarkTextHeightLimit(float value) {
+    m_untilMarkTextHeightLimit = value;
+    if (m_config) {
+        m_config->setValue(ConfigKey("[Waveform]", "UntilMarkTextHeightLimit"),
+                toUntilMarkTextHeightLimitIndex(m_untilMarkTextHeightLimit));
+    }
+}
+
+// static
+Qt::Alignment WaveformWidgetFactory::toUntilMarkAlign(int index) {
+    switch (index) {
+    case 0:
+        return Qt::AlignTop;
+    case 1:
+        return Qt::AlignVCenter;
+    case 2:
+        return Qt::AlignBottom;
+    }
+    assert(false);
+    return Qt::AlignVCenter;
+}
+// static
+int WaveformWidgetFactory::toUntilMarkAlignIndex(Qt::Alignment align) {
+    switch (align) {
+    case Qt::AlignTop:
+        return 0;
+    case Qt::AlignVCenter:
+        return 1;
+    case Qt::AlignBottom:
+        return 2;
+    default:
+        break;
+    }
+    assert(false);
+    return 1;
+}
+// static
+float WaveformWidgetFactory::toUntilMarkTextHeightLimit(int index) {
+    switch (index) {
+    case 0:
+        return 0.333f;
+    case 1:
+        return 1.f;
+    }
+    assert(false);
+    return 0.33f;
+}
+// static
+int WaveformWidgetFactory::toUntilMarkTextHeightLimitIndex(float value) {
+    if (value == 0.333f) {
+        return 0;
+    }
+    if (value == 1.f) {
+        return 1;
+    }
+    assert(false);
+    return 0;
+}
